@@ -82,6 +82,31 @@ test_that("partial_residuals() gives correct results for GLMs", {
                wt_out - mean(wt_out))
 })
 
+test_that("partial_residuals() handles offsets", {
+  # simple offset
+  fit <- lm(mpg ~ hp, offset = qsec, data = mtcars)
+
+  out <- partial_residuals(fit) |>
+    pull(.partial_resid)
+
+  pr <- residuals(fit, type = "partial")
+  pr <- pr[, "hp"] - mean(pr[, "hp"])
+
+  # as in previous test, centered values should match
+  expect_equal(out - mean(out), unname(pr))
+
+  # transformed offset
+  fit <- lm(mpg ~ hp, offset = log(qsec), data = mtcars)
+
+  out <- partial_residuals(fit) |>
+    pull(.partial_resid)
+
+  pr <- residuals(fit, type = "partial")
+  pr <- pr[, "hp"] - mean(pr[, "hp"])
+
+  expect_equal(out - mean(out), unname(pr))
+})
+
 test_that("binned_residuals() produces correct amount of data", {
   fit <- lm(mpg ~ hp + qsec, data = mtcars)
 
@@ -115,6 +140,15 @@ test_that("augment_longer() produces correct amount of data", {
 
   out <- augment_longer(fit)
 
+  expect_equal(nrow(out), nrow(mtcars) * 3)
+  expect_setequal(unique(out$.predictor_name),
+                  c("cyl", "disp", "hp"))
+
+  # one row per predictor, not per regressor:
+  fit <- lm(mpg ~ cyl + poly(disp, 2) + hp, data = mtcars)
+
+  out <- augment_longer(fit)
+
   expect_equal(nrow(out), nrow(mtcars) * 3)
   expect_setequal(unique(out$.predictor_name),
                   c("cyl", "disp", "hp"))
